package com.reyun.report.roi;

import com.google.common.base.Function;
import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ListMultimap;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.reyun.model.ReportListResult;
import com.reyun.report.AbstractReportParse;
import com.reyun.util.Constant;
import org.joda.time.DateTime;
import org.joda.time.Interval;
import org.joda.time.Period;
import org.joda.time.PeriodType;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

import static com.google.common.base.Preconditions.checkArgument;

/**
 * Created by nolan on 15/02/2017.
 * description:
 */
public class ListRoiByDayParse extends AbstractReportParse {
	private final String DATEFORMAT = "yyyy-MM-dd";
	public enum  EnumRangeType {
		DateType, INTERVALType;
	}

	public ReportListResult parse(ReportListResult reportResult, Long app, Long account, String reportName, Map<String, String> conditions) {
		checkArgument(conditions.get("install_startdate")!=null, "激活开始时间不允许为空");
		checkArgument(conditions.get("install_enddate")!=null, "激活结束时间不允许为空");

		DateTime install_startdate = new DateTime(conditions.get("install_startdate"));
		DateTime install_enddate = new DateTime(conditions.get("install_enddate"));

		if(!conditions.containsKey("interval")){
			checkArgument(conditions.get("pay_startdate")!=null, "付费开始时间不允许为空");
			checkArgument(conditions.get("pay_enddate")!=null, "付费结束时间不允许为空");

			DateTime pay_startdate = new DateTime(conditions.get("pay_startdate"));
			DateTime pay_enddate = new DateTime(conditions.get("pay_enddate"));

			return handleDateRange(reportResult, install_startdate, install_enddate, pay_startdate, pay_enddate);
		}else {
			checkArgument(conditions.get("interval")!=null, "interval不允许为空");

			return handlePayInterval(reportResult, install_startdate, install_enddate, Integer.parseInt(conditions.get("interval")));
		}
	}


	/*{
		"help" : [ "" ],
		"name" : [ "日期", "付费日期", "激活设备数", "新增用户付费人数", "新增用户付费金额" ],
		"column" : [ "ds", "pay_ds", "num_ins", "num_pay", "amt_pay" ],
		"key" : null,
			"val" : [ {
		"num_ins" : 0,
				"isall0" : false,
				"pay_ds" : "2017-02-16",
				"amt_pay" : 846.0,
				"num_pay" : 1,
				"ds" : "2017-01-05"
	}, {
		"num_ins" : 0,
				"isall0" : false,
				"pay_ds" : "2017-02-16",
				"amt_pay" : 24.0,
				"num_pay" : 1,
				"ds" : "2017-01-16"
	} ],
		"columnkey" : [ "ds", "pay_ds", "num_ins", "num_pay", "amt_pay" ]
	}

	*/
	private ReportListResult handleDateRange(ReportListResult reportResult, DateTime installStartDate, DateTime installEndDate, DateTime payStartDate, DateTime payEndDate){
		//1. 报表名称及columnkey填充
		Period period = new Period(installStartDate, installEndDate, PeriodType.days());
		int installNum = period.getDays() + 1;
		period = new Period(payStartDate, payEndDate, PeriodType.days());
		int payNum = period.getDays() + 1;

		reportResult.getName().remove("付费日期");
		reportResult.getName().remove("Payment Date");//Payment Date是付费日期的国际化


		reportResult.getColumnkey().remove("pay_ds");
		for (int i = 0; i < payNum; i++) {
			reportResult.getName().add(payStartDate.plusDays(i).toString(DATEFORMAT)+"_新增用户付费金额");
			reportResult.getName().add(payStartDate.plusDays(i).toString(DATEFORMAT)+"_新增用户付费人数");

			reportResult.getColumnkey().add(payStartDate.plusDays(i).toString(DATEFORMAT)+"_amt_pay");
			reportResult.getColumnkey().add(payStartDate.plusDays(i).toString(DATEFORMAT)+"_num_pay");
		}

		//2. 将同一激活时间的不同付费日期进行数据汇总并自动填充
		//2.1 数据汇总
		ListMultimap<String, Map<String, Object>> dsMultiMap = ArrayListMultimap.create();
		for (Map<String, Object> tmpKey : reportResult.getVal()) {
			dsMultiMap.put(String.valueOf(tmpKey.get("ds")), tmpKey);
		}
		//2.2 合并行数据
		List<Map<String, Object>> transformList = Lists.newArrayList();
		for (int i = 0; i < installNum; i++) {
			Map<String, Object> transItem = Maps.newHashMap();
			String ds = installStartDate.plusDays(i).toString(DATEFORMAT);
			transItem.put("ds", ds);

			if(dsMultiMap.containsKey(ds)){
				List<Map<String, Object>> rowItem = dsMultiMap.get(ds);
				Map<String,Map<String, Object>> payRowItemMap  = Maps.uniqueIndex(rowItem, new Function<Map<String, Object>, String>() {
					@Override
					public String apply(Map<String, Object> stringObjectMap) {
						return String.valueOf(stringObjectMap.get("pay_ds"));
					}
				});
				Map<String, Object> numInsItem = payRowItemMap.get("1970-01-01");
				Long num_ins = 0l;
				if (numInsItem != null) {
					num_ins = fixNull(numInsItem.get("num_ins"));
				}
				transItem.put("num_ins", num_ins);
				double total_amt_pay = 0d;
				int total_num_pay = 0;
				//2.2 列付费区间不存在的日期进行数据补0
				for (int j = 0; j < payNum; j++) {
					String payDs = payStartDate.plusDays(j).toString(DATEFORMAT);
					double itemAmtPay = 0d;
					long itemNumPay = 0l;
					Map payItem = payRowItemMap.get(payDs);
					if (payItem != null) {
						itemAmtPay = fixNull(payItem.get("amt_pay"), BigDecimal.ZERO);
						itemNumPay = fixNull(payItem.get("num_pay"));
					}
					total_amt_pay += itemAmtPay;
					total_num_pay += itemNumPay;

					transItem.put(payDs+"_amt_pay", itemAmtPay);
					transItem.put(payDs+"_num_pay", itemNumPay);
				}
				transItem.put("amt_pay", total_amt_pay);
				transItem.put("num_pay", total_num_pay);
				transItem.put("isall", total_amt_pay == 0 && total_num_pay == 0 && num_ins == 0);
			}else{
				//3. 将激活区间内不存在的日期进行数据补0
				transItem.put("num_ins", 0);
				transItem.put("amt_pay", 0d);
				transItem.put("num_pay", 0);
				for (int j = 0; j < payNum; j++) {
					final String tmpPayDate = payStartDate.plusDays(j).toString(DATEFORMAT);
					transItem.put(tmpPayDate +"_amt_pay", 0d);
					transItem.put(tmpPayDate +"_num_pay", 0);
				}
				transItem.put("isall", true);
			}
			transformList.add(transItem);
		}
		reportResult.setVal(transformList);
		return reportResult;
	}

	private ReportListResult handlePayInterval(ReportListResult reportResult, DateTime installStartDate, DateTime installEndDate, Integer interval){
		//1. 报表名称及columnkey填充
		Period period = new Period(installStartDate, installEndDate, PeriodType.days());
		int installNum = period.getDays() + 1;
		int payNum = period.getDays() + interval + 1;

		if(reportResult.getName().size()>0){
			reportResult.getName().remove("付费日期");
			reportResult.getColumnkey().remove("pay_ds");
		}
		for (int i = 0; i < payNum; i++) {
			final String tmpDate = installStartDate.plusDays(i).toString(DATEFORMAT);
			reportResult.getName().add(tmpDate +"_新增用户付费金额");
			reportResult.getName().add(tmpDate +"_新增用户付费人数");

			reportResult.getColumnkey().add(tmpDate +"_amt_pay");
			reportResult.getColumnkey().add(tmpDate +"_num_pay");
		}

		//2. 将同一激活时间的不同付费日期进行数据汇总并自动填充
		//2.1 数据汇总
		ListMultimap<String, Map<String, Object>> dsMultiMap = ArrayListMultimap.create();
		for (Map<String, Object> tmpKey : reportResult.getVal()) {
			dsMultiMap.put(String.valueOf(tmpKey.get("ds")), tmpKey);
		}
		//2.2 合并行数据
		List<Map<String, Object>> transformList = Lists.newArrayList();
		for (int i = 0; i < installNum; i++) {
			Map<String, Object> transItem = Maps.newHashMap();
			DateTime dsDate = installStartDate.plusDays(i);
			String ds = dsDate.toString(DATEFORMAT);
			transItem.put("ds", ds);

			if(dsMultiMap.containsKey(ds)){
				List<Map<String, Object>> rowItem = dsMultiMap.get(ds);
				Map<String,Map<String, Object>> payRowItemMap  = Maps.uniqueIndex(rowItem, new Function<Map<String, Object>, String>() {
					@Override
					public String apply(Map<String, Object> stringObjectMap) {
						return String.valueOf(stringObjectMap.get("pay_ds"));
					}
				});
				Map<String, Object> numInsItem = payRowItemMap.get("1970-01-01");
				Long num_ins = 0l;
				if (numInsItem != null) {
					num_ins = fixNull(numInsItem.get("num_ins"));
				}
				transItem.put("num_ins", num_ins);
				double total_amt_pay = 0d;
				int total_num_pay = 0;
				//2.2 列付费区间不存在的日期进行数据补0
				for (int j = 0; j < payNum; j++) {
					final DateTime payDsDate = dsDate.plusDays(j);
					String payDs = payDsDate.toString(DATEFORMAT);

					boolean isContain = new Interval(dsDate, dsDate.plusDays(interval + 1)).contains(payDsDate);
					if(isContain){
						 double itemAmtPay = 0d;
						 long itemNumPay = 0l;
						 Map payItem = payRowItemMap.get(payDs);
						 if (payItem != null) {
							 itemAmtPay = fixNull(payItem.get("amt_pay"), BigDecimal.ZERO);
							 itemNumPay = fixNull(payItem.get("num_pay"));
						 }
						 total_amt_pay += itemAmtPay;
						 total_num_pay += itemNumPay;

						transItem.put(payDs+"_amt_pay", itemAmtPay);
						transItem.put(payDs+"_num_pay", itemNumPay);
					}else{
						transItem.put(payDs+"_amt_pay", "");
						transItem.put(payDs+"_num_pay", "");
					}
				}
				transItem.put("amt_pay", total_amt_pay);
				transItem.put("num_pay", total_num_pay);
				transItem.put("isall", total_amt_pay == 0 && total_num_pay == 0 && num_ins == 0);
			}else{
				//3. 将激活区间内不存在的日期进行数据补0
				transItem.put("num_ins", 0);
				transItem.put("amt_pay", 0d);
				transItem.put("num_pay", 0);
				for (int j = 0; j < payNum; j++) {
					final DateTime payDsDate = dsDate.plusDays(j);
					String payDs = payDsDate.toString(DATEFORMAT);

					boolean isContain = new Interval(dsDate, dsDate.plusDays(interval + 1)).contains(payDsDate);
					if(isContain){
						transItem.put(payDs+"_amt_pay", 0d);
						transItem.put(payDs+"_num_pay", 0);
					}else{
						transItem.put(payDs+"_amt_pay", "");
						transItem.put(payDs+"_num_pay", "");
					}
				}
				transItem.put("isall", true);
			}
			transformList.add(transItem);
		}
		reportResult.setVal(transformList);
		return reportResult;
	}

	private Long fixNull(Object val) {
		return val != null ? Long.parseLong(String.valueOf(val)) : 0;
	}

	private Double fixNull(Object val, BigDecimal defaultVal) {
		return val != null ? new BigDecimal(String.valueOf(val)).setScale(Constant.FMT_NEWSCALE, Constant.FMT_ROUNDINGMODE).doubleValue() : defaultVal.doubleValue();
	}

}
