package com.reyun.util;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Date;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.collect.Lists;
import com.qcloud.cos.COSClient;
import com.qcloud.cos.ClientConfig;
import com.qcloud.cos.auth.BasicCOSCredentials;
import com.qcloud.cos.auth.COSCredentials;
import com.qcloud.cos.http.HttpMethodName;
import com.qcloud.cos.model.COSObjectSummary;
import com.qcloud.cos.model.GeneratePresignedUrlRequest;
import com.qcloud.cos.model.ListObjectsRequest;
import com.qcloud.cos.model.ObjectListing;
import com.qcloud.cos.model.ObjectMetadata;
import com.qcloud.cos.region.Region;

/* 腾迅云对象存储工具类*/
public class CosUtil {
    private static final Logger LOG = LoggerFactory.getLogger(CosUtil.class);
    private static CosUtil cosUtil = new CosUtil();
    private COSClient cosClient;
    private static final String BucketNamePostfix = "-" + Constant.cosAppid;
    
    private CosUtil() {
        COSCredentials cred = new BasicCOSCredentials(Constant.cosSecretId, Constant.cosSecretKey);
        ClientConfig clientConfig = new ClientConfig(new Region(Constant.cosRegion));
        cosClient = new COSClient(cred, clientConfig);
    }

    public static CosUtil getInstance() {
        return cosUtil;
    }

    private String formatBucketName(String bucketName) {
        return bucketName.endsWith(BucketNamePostfix) ? bucketName : bucketName + BucketNamePostfix;
    }

    public List<String> listObjects(String bucketName, String prefix) {
        List<String> rtnList = Lists.newArrayList();
        ListObjectsRequest listObjectsRequest = new ListObjectsRequest();
        listObjectsRequest.setBucketName(formatBucketName(bucketName));
        listObjectsRequest.setPrefix(prefix);
        //listObjectsRequest.setMaxKeys(1000);
        ObjectListing objectListing = cosClient.listObjects(listObjectsRequest);
        List<COSObjectSummary> cosObjectSummaries = objectListing.getObjectSummaries();
        for (COSObjectSummary cosObjectSummary : cosObjectSummaries) {
            rtnList.add(cosObjectSummary.getKey());
        }
        return rtnList;
    }

    public long putObjectInStream(String bucketName, String key, InputStream in, String contentType, long contentLength)
            throws IOException {
        ObjectMetadata metadata = new ObjectMetadata();
        if (contentType != null) {
            metadata.setContentType(contentType);
        }
        metadata.setContentLength(contentLength);
        cosClient.putObject(formatBucketName(bucketName), key, in, metadata);
        return contentLength;
    }

    public long putObjectInString(String bucketName, String key, String str) throws IOException {
        final byte[] bytes = str.getBytes("UTF-8");
        BufferedInputStream bis = new BufferedInputStream(new ByteArrayInputStream(bytes));
        try {
            return putObjectInStream(formatBucketName(bucketName), key, bis, "text/plain; charset=UTF-8", bytes.length);
        } finally {
            IOUtil.close(bis);
        }
    }

    public byte[] getObjectContentByteArray(String bucketName, String key) throws IOException {
        InputStream is = null;
        ByteArrayOutputStream bos = null;
        try {
            is = getObjectContentStream(formatBucketName(bucketName), key);
            bos = new ByteArrayOutputStream();
            IOUtil.copy(is, bos);
            return bos.toByteArray();
        } finally {
            if (bos != null) {
                IOUtil.close(bos);
            }
            if (is != null) {
                IOUtil.close(is);
            }
        }
    }

    public InputStream getObjectContentStream(String bucketName, String key) {
        return cosClient.getObject(formatBucketName(bucketName), key).getObjectContent();
    }

    public void deleteObject(String bucketName, String key) {
        cosClient.deleteObject(formatBucketName(bucketName), key);
    }

    public long getLength(String bucketName, String key) {
        ObjectMetadata om = cosClient.getObjectMetadata(formatBucketName(bucketName), key);
        return om == null ? 0 : om.getContentLength();
    }

    public boolean doesObjectExist(String bucketName, String objectName) {
        return cosClient.doesObjectExist(formatBucketName(bucketName), objectName);
    }

    public URL generatePresignedUrl(String bucketName, String objectName, long expireMsFromCurrent) {
        GeneratePresignedUrlRequest req = new GeneratePresignedUrlRequest(formatBucketName(bucketName), objectName,
                HttpMethodName.GET);
        Date expirationDate = new Date(System.currentTimeMillis() + expireMsFromCurrent);
        req.setExpiration(expirationDate);
        return cosClient.generatePresignedUrl(req);
    }
}