package com.reyun.util;

import org.apache.kafka.clients.producer.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.*;

/**
 * kafka工具类
 * 
 * @author liruijie@reyun.com
 * @date 2015年7月17日
 */
public class KafkaUtil {
    private static final Logger logger = LoggerFactory.getLogger("stdout");
    private static KafkaUtil instance;
    private static final ThreadPoolExecutor exec = (ThreadPoolExecutor) Executors.newFixedThreadPool(2);
    private static Properties props;
    private static Map<String, Producer<String, String>> producers = new ConcurrentHashMap<String, Producer<String, String>>();
    private static boolean enableKafka = true;

    private KafkaUtil() {
    }

    public static final KafkaUtil getInstance() {
        if (null == instance) {
            syncInit();
        }
        return instance;
    }

    private static synchronized void syncInit() {
        if (instance == null) {
            if (!ValidateUtil.isValid(Constant.kafkaBroker)) {
                enableKafka = false;
            }
            props = new Properties();
            props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, Constant.kafkaBroker);
            props.put("key.serializer", "org.apache.kafka.common.serialization.StringSerializer");
            props.put("value.serializer", "org.apache.kafka.common.serialization.StringSerializer");
            props.put(ProducerConfig.ACKS_CONFIG, "1");
            props.put("retries", 1);
            if (Constant.isAwsEnv()) {
                props.put(ProducerConfig.COMPRESSION_TYPE_CONFIG, "gzip");
            } else if (Constant.isQcloudEnv()) {
//                props.put(ProducerConfig.COMPRESSION_TYPE_CONFIG, "snappy");
            }
            props.put("batch.size", 262144); // unit: byte
            props.put("linger.ms", 200);
            // props.put("partitioner.class", "kafka.producer.DefaultPartitioner"); // based on the hash of the key
            ScheduledExecutorService scheduledThreadPool = Executors.newScheduledThreadPool(1);
            scheduledThreadPool.scheduleWithFixedDelay(new Runnable() {

                @Override
                public void run() {
                    logger.info("kafka thread pool: active count: {}, pool size: {}, queue size: {}",
                            exec.getActiveCount(), exec.getPoolSize(), exec.getQueue().size());
                }
            }, 60, 300, TimeUnit.SECONDS);

            instance = new KafkaUtil();
        }
    }

    /**
     * 发送数据到对应的topic
     *
     * @param topic
     * @param data
     */
    public void send(final String topic, final String data) {
        send(topic, null, data);
    }

    public void send(final String topic, final String key, final String data) {
        if (enableKafka){
            exec.execute(new Runnable() {
                @Override
                public void run() {
                    try {
                        Producer<String, String> producer = producers.get(topic);
                        if (null == producer) {
                            producer = new KafkaProducer<String, String>(props);
                            producers.put(topic, producer);
                        }
                        ProducerRecord<String, String> message = new ProducerRecord<String, String>(topic, key, data);
                        producer.send(message, new Callback() {
                            @Override
                            public void onCompletion(RecordMetadata metadata, Exception exception) {
                                if (exception != null) {
                                    logger.info("metadata:" + metadata + ", exception:" + exception.getMessage(), exception);
                                }
                            }
                        });
                    } catch (Exception e) {
                        logger.error(e.getMessage(), e);
                    }
                }
            });
        }
    }
}