package com.reyun.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.bson.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.mongodb.client.FindIterable;
import com.mongodb.client.MongoClient;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoCursor;
import com.mongodb.client.model.BulkWriteOptions;
import com.mongodb.client.model.Filters;
import com.mongodb.client.model.ReplaceOneModel;
import com.mongodb.client.model.ReplaceOptions;
import com.mongodb.client.model.UpdateOptions;
import com.mongodb.client.model.Updates;
import com.mongodb.client.model.WriteModel;
import com.mongodb.client.result.DeleteResult;
import com.mongodb.client.result.UpdateResult;
import com.reyun.util.NoSQLDBUtil.Result;

public class MongoDBUtil {
    private static final Logger logger = LoggerFactory.getLogger(MongoDBUtil.class);
    private static MongoDBUtil instance;
    private static MongoClient mongoClient;
    private static String dbName = "tkio";

    public static MongoDBUtil getInstance() {
        if (null == instance) {
            syncInit();
        }
        return instance;
    }

    private static synchronized void syncInit() {
        if (instance == null) {
            mongoClient = MongoClients.create(Constant.MONGO_URI);
            instance = new MongoDBUtil();
        }
    }

    private MongoDBUtil() {
    }

    public UpdateResult writeItem(String tableName, Document document) {
        MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
        return collection.replaceOne(Filters.eq("_id", document.getString("_id")), document,
                new ReplaceOptions().upsert(true));
    }

    public DeleteResult deleteItem(String tableName, String primaryKeyValue) {
        MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
        return collection.deleteOne(Filters.eq("_id", primaryKeyValue));
    }

    public UpdateResult updateItem(String tableName, String primaryKeyName, String primaryKeyValue, Document document) {
        MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
        return collection.updateOne(Filters.eq("_id", primaryKeyValue), document,
                new UpdateOptions().upsert(true));
    }

    public Document queryDocumentByPrimaryKey(String tableName, String primaryKeyName, String primaryKeyValue) {
        MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
        return collection.find(Filters.eq("_id", primaryKeyValue)).first();
    }

    /**
     * 批量读
     * mongodb primary key固定为_id，所以此处忽略primaryKeyName入参
     */
    public List<Document> batchQueryByPrimaryKey(String tableName, String primaryKeyName, int batchSize,
            List<String> values) {
        List<Document> result = new ArrayList<>();
        MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
        // batchSize 设置每次取出数据量
        FindIterable<Document> findIterable = collection.find(Filters.in("_id", values)).batchSize(batchSize);
        MongoCursor<Document> mongoCursor = findIterable.iterator();
        while (mongoCursor.hasNext()) {
            Document doc = mongoCursor.next();
            doc.put(primaryKeyName, doc.get("_id"));
            result.add(doc);
        }
        mongoCursor.close();
        return result;
    }

    /**
     * 批量写
     * @param tableName
     * @param items
     */
    public void batchWrite(String tableName, List<Result> items) {
        try {
            List<WriteModel<Document>> writeModelList = new ArrayList<WriteModel<Document>>();

            MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
            for (Result obj : items) {
                if (obj != null && obj.getResult() != null){
                    Document document = (Document) obj.getResult();
                    ReplaceOneModel<Document> updateOneModel = new ReplaceOneModel<Document>(
                            Filters.eq("_id", document.getString("_id")), document, new ReplaceOptions().upsert(true));
                    writeModelList.add(updateOneModel);
                }
            }
            if (writeModelList.size() > 0){
                collection.bulkWrite(writeModelList, new BulkWriteOptions().ordered(false));
            }
        } catch (Exception e) {
            logger.error("MongoDBUtil BatchWrite error : " + e.getMessage(), e);
        }
    }

    public void increase(String tableName, String primaryKeyName, String primaryKeyValue, String field, int incValue) {
        MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
        collection.findOneAndUpdate(Filters.eq("_id", primaryKeyValue), new Document("$inc", new Document(field, incValue)));
    }

    public MongoCollection<Document> getCollection(String tableName) {
        return mongoClient.getDatabase(dbName).getCollection(tableName);
    }

    @SuppressWarnings("unchecked")
    public Set<String> getStringSet(String tableName, String primaryKeyValue, String setFieldName) {
        Set<String> set = new HashSet<>();
        Document document = queryDocumentByPrimaryKey(tableName, "_id", primaryKeyValue);
        if (document != null) {
            set.addAll((List<String>)document.get(setFieldName));
        }
        return set;
    }

    public void writeStringSet(String tableName, String primaryKeyValue, String field, Set<String> set) {
        MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
        collection.updateOne(Filters.eq("_id", primaryKeyValue), Updates.set(field, set),
                new UpdateOptions().upsert(true));
    }

    @SuppressWarnings("rawtypes")
    public void writeList(String tableName, String primaryKeyValue, String field, List list) {
        MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
        collection.updateOne(Filters.eq("_id", primaryKeyValue), Updates.set(field, list),
                new UpdateOptions().upsert(true));
    }

    @SuppressWarnings({ "rawtypes" })
    public List getList(String tableName, String primaryKeyValue, String listFieldName) {
        List list = new ArrayList<>();
        Document document = queryDocumentByPrimaryKey(tableName, "_id", primaryKeyValue);
        if (document != null) {
            return (List) document.get(listFieldName);
        }
        return list;
    }

    @SuppressWarnings("rawtypes")
    public void writeMap(String tableName, String primaryKeyValue, String field, Map map) {
        MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
        collection.updateOne(Filters.eq("_id", primaryKeyValue), Updates.set(field, map),
                new UpdateOptions().upsert(true));
    }

    @SuppressWarnings({ "rawtypes" })
    public Map getMap(String tableName, String primaryKeyValue, String listFieldName) {
        Map map = new HashMap<>();
        Document document = queryDocumentByPrimaryKey(tableName, "_id", primaryKeyValue);
        if (document != null) {
            return (Map) document.get(listFieldName);
        }
        return map;
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    public void appendSetData(String tableName, String primaryKeyValue, List<String> nestedColumnName, Set set) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < nestedColumnName.size(); i++) {
            if (sb.length() > 0) {
                sb.append(".");
            }
            sb.append(nestedColumnName.get(i));
        }
        MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
        collection.updateOne(Filters.eq("_id", primaryKeyValue),
                Updates.addEachToSet(sb.toString(), new ArrayList<>(set)), new UpdateOptions().upsert(true));
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    public void deleteDataFromSet(String tableName, String primaryKeyValue, List<String> nestedColumnName, Set set) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < nestedColumnName.size(); i++) {
            if (sb.length() > 0) {
                sb.append(".");
            }
            sb.append(nestedColumnName.get(i));
        }
        MongoCollection<Document> collection = mongoClient.getDatabase(dbName).getCollection(tableName);
        collection.updateOne(Filters.eq("_id", primaryKeyValue), Updates.pullAll(sb.toString(), new ArrayList<>(set)));
    }
}
